__project_name__ = "feathercarver"
from .argument_parser import ArgumentParser
from .file_processor import FileProcessor
from .link_fixer import LinkFixer


def main() -> int:
    parser = ArgumentParser()
    file_paths = parser.parse_arguments()

    link_fixer = LinkFixer()
    file_processor = FileProcessor(link_fixer)

    for file_path in file_paths:
        file_processor.process_file(file_path)

    return 0


__all__ = ["main"]



import re


class LinkFixer:
    def __init__(self):
        self.pattern = re.compile(
            r"""
          \[              # Opening square bracket
          (?P<text>       # Named capturing group for link text
              [\s\S]*?    # Any characters (including newlines), non-greedy
          )               # End named capturing group for link text
          \]              # Closing square bracket
          \s*             # Optional whitespace
          \(              # Opening parenthesis
          \s*             # Optional whitespace
          (?P<url>        # Named capturing group for URL
              (?:         # Non-capturing group for URL content
                  \S      # Non-whitespace character
                  (?:     # Non-capturing group
                      \s?     # Optional single whitespace
                      \S      # Non-whitespace character
                  )*      # Zero or more repetitions
              )+          # One or more repetitions of the URL content
          )               # End named capturing group for URL
          (?:             # Non-capturing group for optional title
              \s+         # Required whitespace before title
              (?P<title>  # Named capturing group for title
                  ["']    # Opening quote (single or double)
                  .*?     # Any characters, non-greedy
                  ["']    # Closing quote (single or double)
              )           # End named capturing group for title
          )?              # Title is optional
          \s*             # Optional whitespace
          \)              # Closing parenthesis
      """,
            re.VERBOSE | re.DOTALL,
        )

        self.block_pattern = re.compile(
            r"""
          (?P<code>           # Named capturing group for code blocks
              (?:             # Non-capturing group for block or inline code
                  ```[\s\S]*?```  # Block code
                  |               # OR
                  `[^`\n]+?`      # Inline code
              )
          )
      """,
            re.VERBOSE,
        )

    def fix_markdown_links(self, content):
        blocks = self.block_pattern.split(content)

        for i in range(len(blocks)):
            if not self.block_pattern.match(blocks[i]):
                blocks[i] = self.pattern.sub(self._replace_link, blocks[i])

        return "".join(blocks)

    def _replace_link(self, match):
        link_text = re.sub(r"\s+", " ", match.group("text").strip())
        url = re.sub(r"\s+", " ", match.group("url").strip())
        title = match.group("title")
        if title:
            title = re.sub(r"\s+", " ", title.strip())
            return f"[{link_text}]({url} {title})"
        else:
            return f"[{link_text}]({url})"



import sys


class FileProcessor:
    def __init__(self, link_fixer):
        self.link_fixer = link_fixer

    def process_file(self, file_path):
        try:
            with open(file_path, "r") as file:
                content = file.read()

            fixed_content = self.link_fixer.fix_markdown_links(content)

            with open(file_path, "w") as file:
                file.write(fixed_content)
            print(f"Processed {file_path}")
        except IOError as e:
            print(f"Error processing {file_path}: {e}", file=sys.stderr)



import argparse
import typing


class ArgumentParser:
    def parse_arguments(self) -> typing.List[str]:
        parser = argparse.ArgumentParser(
            description="Fix broken Markdown links in files."
        )
        parser.add_argument(
            "files", nargs="+", help="One or more Markdown files to process"
        )
        args = parser.parse_args()
        return list(dict.fromkeys(args.files))
